package com.tzld.piaoquan.longarticle.service.local.impl;

import com.alibaba.fastjson.JSONObject;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.piaoquan.longarticle.model.bo.PublishMiniprogramBo;
import com.tzld.piaoquan.longarticle.model.bo.VideoDetail;
import com.tzld.piaoquan.longarticle.model.po.PlanAccount;
import com.tzld.piaoquan.longarticle.model.po.PublishContent;
import com.tzld.piaoquan.longarticle.model.po.PublishMiniprogram;
import com.tzld.piaoquan.longarticle.service.local.CardService;
import com.tzld.piaoquan.longarticle.service.remote.AigcService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.*;

@Slf4j
@Service
public class CardServiceImpl implements CardService {

    @ApolloJsonValue("${miniMap:{}}")
    private JSONObject miniProgramMap;

    @Autowired
    AigcService aigcService;


    public JSONObject createGzhPath(String videoId, String sharedUid, String ghId, Integer miniProgramType) throws UnsupportedEncodingException {
        String rootShareId = UUID.randomUUID().toString();
        String rootSourceId;
        switch (miniProgramType) {
            case 1:
                rootSourceId = "longArticles_" + generateSourceId();
                break;
            case 2:
                rootSourceId = "touliu_tencentGzhArticle_" + ghId + "_" + generateSourceId();
                break;
            case 3:
                rootSourceId = "WeCom_" + generateSourceId();
                break;
            default:
                rootSourceId = "Error mini_program_type " + miniProgramType;
        }

        String url = String.format("pages/user-videos?id=%s&su=%s&fromGzh=1&rootShareId=%s&shareId=%s&rootSourceId=%s",
                videoId, sharedUid, rootShareId, rootShareId, rootSourceId);
        String productionPath = String.format("pages/category?jumpPage=%s", URLEncoder.encode(url, StandardCharsets.UTF_8.toString()));
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("rootShareId", rootShareId);
        jsonObject.put("rootSourceId", rootSourceId);
        jsonObject.put("productionPath", productionPath);
        return jsonObject;
    }

    private String generateSourceId() {
        long timestamp = System.currentTimeMillis();
        int randomNum = new Random().nextInt(9000) + 1000;
        String hashInput = timestamp + "-" + randomNum;
        return md5(hashInput);
    }

    private String md5(String input) {
        try {
            MessageDigest md = MessageDigest.getInstance("MD5");
            byte[] messageDigest = md.digest(input.getBytes(StandardCharsets.UTF_8));
            StringBuilder sb = new StringBuilder();
            for (byte b : messageDigest) {
                sb.append(String.format("%02x", b));
            }
            return sb.toString();
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        }
    }

    public PublishMiniprogramBo generateSingleCard(Integer index, Integer miniId, VideoDetail videoDetail,
                                                   PlanAccount planAccount, PublishContent publishContent) throws UnsupportedEncodingException {
        String strMiniId = String.valueOf(miniId);
        JSONObject miniInfo = miniProgramMap.getJSONObject(strMiniId);
        String avatar = miniInfo.getString("avatar");
        String appId = miniInfo.getString("id");
        String appName = miniInfo.getString("name");

        String videoId = videoDetail.getVideoId();
        String uid = videoDetail.getUid();
        JSONObject jsonObject = createGzhPath(videoId, uid, planAccount.getGhId(), planAccount.getMiniprogramUseType());

        PublishMiniprogramBo publishMiniprogramBo = new PublishMiniprogramBo();
        publishMiniprogramBo.setContentId(publishContent.getId());
        publishMiniprogramBo.setPlanAccountId(planAccount.getId());
        publishMiniprogramBo.setAppId(appId);
        publishMiniprogramBo.setPublishContentId(publishContent.getPublishContentId());
        publishMiniprogramBo.setAppName(appName);
        publishMiniprogramBo.setAvatar(avatar);
        publishMiniprogramBo.setMiniId(miniId);
        publishMiniprogramBo.setMiniProgramType(planAccount.getMiniprogramUseType());
        publishMiniprogramBo.setVideoId(Long.parseLong(videoId));
        publishMiniprogramBo.setSource(videoDetail.getSource());
        publishMiniprogramBo.setVideoTitle(videoDetail.getKimiTitle());
        publishMiniprogramBo.setVideoCover(videoDetail.getVideoCover());
        publishMiniprogramBo.setVideoPath(videoDetail.getVideoPath());
        publishMiniprogramBo.setProductionPath(jsonObject.getString("productionPath"));
        publishMiniprogramBo.setRootSourceId(jsonObject.getString("rootSourceId"));
        publishMiniprogramBo.setRootShareId(jsonObject.getString("rootShareId"));
        publishMiniprogramBo.setIndex(index);
        publishMiniprogramBo.setTraceId(videoDetail.getTraceId());
        publishMiniprogramBo.setCrawlerVideoId(videoDetail.getCrawlerVideoId());
        publishMiniprogramBo.setVideoOssPath(videoDetail.getVideoOss());
        publishMiniprogramBo.setUserId(videoDetail.getUid());
        return publishMiniprogramBo;
    }

    public List<PublishMiniprogramBo> generateCards(List<VideoDetail> videoDetails, PlanAccount planAccount, PublishContent publishContent) {
        try {
            int longArticlesMiniProgramId = 25;
            int touliuMiniProgramId = 33;
            int weComMiniProgramId = 27;
            int miniId = 0;
            List<PublishMiniprogramBo> cardList = new ArrayList<>();
            switch (planAccount.getMiniprogramUseType()) {
                case 1:
                    miniId = longArticlesMiniProgramId;
                    break;
                case 2:
                    miniId = touliuMiniProgramId;
                    break;
                case 3:
                    miniId = weComMiniProgramId;
                    break;
                default:
                    break;
            }
            if (miniId == 0) {
                return null;
            }
            for (int index = 0; index < videoDetails.size(); index++) {
                VideoDetail videoDetail = videoDetails.get(index);
                PublishMiniprogramBo card = generateSingleCard(index + 1, miniId, videoDetail, planAccount, publishContent);
                if (card != null) {
                    String videoCover = card.getVideoCover();
                    String wxUrl = aigcService.pushCover(videoCover, card.getPublishContentId());
                    if (StringUtils.isEmpty(wxUrl)) {
                        continue;
                    }
                    card.setWxUrl(wxUrl);
                    cardList.add(card);
                }
            }
            return cardList;
        } catch (UnsupportedEncodingException e) {
            log.error("generateCards error {}", e.getMessage());
        }
        return null;
    }
}
