package com.tzld.piaoquan.longarticle.service.local.impl;

import com.alibaba.fastjson.JSON;
import com.tzld.piaoquan.longarticle.common.constants.TimeConstant;
import com.tzld.piaoquan.longarticle.common.enums.ContentStatusEnum;
import com.tzld.piaoquan.longarticle.common.enums.PublishGzhPushTypeEnum;
import com.tzld.piaoquan.longarticle.dao.mapper.PlanAccountMapper;
import com.tzld.piaoquan.longarticle.dao.mapper.PublishContentMapper;
import com.tzld.piaoquan.longarticle.dao.mapper.PublishMiniprogramMapper;
import com.tzld.piaoquan.longarticle.dao.mapper.RootSourceMapper;
import com.tzld.piaoquan.longarticle.model.bo.MatchContent;
import com.tzld.piaoquan.longarticle.model.bo.PublishMiniprogramBo;
import com.tzld.piaoquan.longarticle.model.bo.VideoDetail;
import com.tzld.piaoquan.longarticle.model.dto.MiniprogramCardRequest;
import com.tzld.piaoquan.longarticle.model.dto.PublishArticleData;
import com.tzld.piaoquan.longarticle.model.po.*;
import com.tzld.piaoquan.longarticle.model.vo.*;
import com.tzld.piaoquan.longarticle.service.local.CoreService;
import com.tzld.piaoquan.longarticle.service.local.PlanAccountService;
import com.tzld.piaoquan.longarticle.service.remote.AigcService;
import com.tzld.piaoquan.longarticle.service.remote.MatchService;
import com.tzld.piaoquan.longarticle.service.remote.VideoService;
import com.tzld.piaoquan.longarticle.service.remote.impl.SortServiceImpl;
import com.tzld.piaoquan.longarticle.utils.DateUtil;
import com.tzld.piaoquan.longarticle.utils.LarkRobotUtil;
import com.tzld.piaoquan.longarticle.utils.TimeZoneUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.time.LocalTime;
import java.util.*;
import java.util.concurrent.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class CoreServiceImpl implements CoreService {

    @Autowired
    private AigcService aigcService;

    @Autowired
    private SortServiceImpl sortService;

    @Autowired
    private ContentServiceImpl contentService;

    @Autowired
    private MatchService matchService;

    @Autowired
    private VideoService videoService;

    @Autowired
    private CardServiceImpl cardService;

    @Autowired
    private PlanAccountMapper planAccountMapper;

    @Autowired
    private PlanAccountService planAccountService;

    @Autowired
    private PublishMiniprogramMapper publishMiniprogramMapper;

    @Autowired
    private PublishContentMapper publishContentMapper;

    @Autowired
    private RootSourceMapper rootSourceMapper;

    @Autowired
    private PublicContentServiceImpl publicContentService;

    @Autowired
    private RootSourceServiceImpl rootSourceService;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Override
    public void initPlanAccount() {
        List<Plan> planList = planAccountService.getPlanList();
        Set<String> planIdSet = planList.stream().map(Plan::getPlanId).collect(Collectors.toSet());
        List<LongArticleSystemPlan> allLongArticleSystemPlans = aigcService.getAllLongArticleSystemPlan();
        for (LongArticleSystemPlan longArticleSystemPlan : allLongArticleSystemPlans) {
            Plan plan = createPlan(longArticleSystemPlan);
            planAccountService.saveOrUpdatePlan(plan);
            planIdSet.remove(longArticleSystemPlan.getId());
            //获取排序策略
            Map<String, String> sortStgMap = new HashMap<>();
            List<GzhArticleSortTaskParam> gzhArticleSortTask = longArticleSystemPlan.getGzhArticleSortTask();
            for (GzhArticleSortTaskParam gzhArticleSortTaskParam : gzhArticleSortTask) {
                String strategy = gzhArticleSortTaskParam.getStrategy();
                List<String> accountIds = gzhArticleSortTaskParam.getAccountIds();
                for (String accountId : accountIds) {
                    sortStgMap.put(accountId, strategy);
                }
            }
            //获取小程序策略
            Map<String, MiniprogramInsertTaskParam> miniStgMap = new HashMap<>();
            List<MiniprogramInsertTaskParam> miniprogramInsertTasks = longArticleSystemPlan.getMiniprogramInsertTasks();
            for (MiniprogramInsertTaskParam miniprogramInsertTaskParam : miniprogramInsertTasks) {
                List<String> accountIds = miniprogramInsertTaskParam.getAccountIds();
                for (String accountId : accountIds) {
                    miniStgMap.put(accountId, miniprogramInsertTaskParam);
                }
            }
            for (LongArticleSystemPlanAccount account : longArticleSystemPlan.getAccountList()) {
                PlanAccount planAccount = new PlanAccount();
                String accountId = account.getId();
                planAccount.setPlanId(longArticleSystemPlan.getId());
                planAccount.setPlanName(longArticleSystemPlan.getName());
                planAccount.setAccountId(accountId);
                planAccount.setAccountName(account.getName());
                planAccount.setGhId(account.getGhId());
                planAccount.setPublishOpenFlag(account.getPublishOpenFlag());
                planAccount.setPublishRate(longArticleSystemPlan.getPublishRate());
                planAccount.setPublishDate(longArticleSystemPlan.getPublishDate());
                planAccount.setPublishNum(longArticleSystemPlan.getPublishNum());
                planAccount.setPublishWindowStart(longArticleSystemPlan.getPublishWindowStart());
                planAccount.setPublishWindowEnd(longArticleSystemPlan.getPublishWindowEnd());
                planAccount.setPublishTimeInterval(longArticleSystemPlan.getPublishTimeInterval());
                planAccount.setPublishNum(longArticleSystemPlan.getPublishNum());
                planAccount.setPublishPreNum(longArticleSystemPlan.getPublishPerNum());
                planAccount.setPublishPreMinNum(longArticleSystemPlan.getPublishPerMinNum());
                planAccount.setPublishPushTime(longArticleSystemPlan.getPublishPushTime());
                planAccount.setPushType(longArticleSystemPlan.getPushType());
                planAccount.setMiniprogramInsertFlag(longArticleSystemPlan.getMiniprogramInsertFlag());
                MiniprogramInsertTaskParam miniprogramInsertTaskParam = miniStgMap.get(accountId);
                if (miniprogramInsertTaskParam != null) {
                    planAccount.setMiniprogramUseType(miniprogramInsertTaskParam.getMiniprogramUseType());
                    planAccount.setMiniStrategy(miniprogramInsertTaskParam.getStrategy());
                }
                if (sortStgMap.containsKey(accountId)) {
                    planAccount.setSortStrategy(sortStgMap.get(accountId));
                }
                planAccount.setPushType(longArticleSystemPlan.getPushType());
                planAccountService.saveOrUpdatePlanAccount(planAccount);
            }
        }
        if (CollectionUtils.isEmpty(planIdSet)) {
            return;
        }
        for (String planId : planIdSet) {
            planAccountService.delPlan(planId);
        }
    }

    private Plan createPlan(LongArticleSystemPlan longArticleSystemPlan) {
        Plan plan = new Plan();
        plan.setPlanId(longArticleSystemPlan.getId());
        plan.setPlanName(longArticleSystemPlan.getName());
        plan.setPublishRate(longArticleSystemPlan.getPublishRate());
        plan.setPublishDate(longArticleSystemPlan.getPublishDate());
        plan.setPublishWindowStart(longArticleSystemPlan.getPublishWindowStart());
        plan.setPublishWindowEnd(longArticleSystemPlan.getPublishWindowEnd());
        plan.setPublishTimeInterval(longArticleSystemPlan.getPublishTimeInterval());
        plan.setPublishNum(longArticleSystemPlan.getPublishNum());
        plan.setPublishPreNum(longArticleSystemPlan.getPublishPerNum());
        plan.setPublishPreMinNum(longArticleSystemPlan.getPublishPerMinNum());
        plan.setPublishPushTime(longArticleSystemPlan.getPublishPushTime());
        plan.setPushType(longArticleSystemPlan.getPushType());
        plan.setMiniprogramInsertFlag(longArticleSystemPlan.getMiniprogramInsertFlag());
        return plan;
    }


    // 定义一个阻塞队列
    private static final ArrayBlockingQueue<MatchContent> queue = new ArrayBlockingQueue<>(100000);

    private static final ArrayBlockingQueue<PlanAccount> queue1 = new ArrayBlockingQueue<>(100000);


    private static final int size = 5;
    // 定义一个线程池，设置消费线程的数量
    private static final ThreadPoolExecutor matchPoolExecutor = (ThreadPoolExecutor) Executors.newFixedThreadPool(size);

    private static final ThreadPoolExecutor corePoolExecutor = (ThreadPoolExecutor) Executors.newFixedThreadPool(10);


    @Override
    public void matchContent() {
        // 启动生产者线程
        new Thread(() -> {
            List<PlanAccount> matchPlanAccounts = planAccountService.getMatchPlanAccount();
            if (CollectionUtils.isEmpty(matchPlanAccounts)) {
                return;
            }
            for (PlanAccount planAccount : matchPlanAccounts) {
                GetContentsParam param = new GetContentsParam();
                param.setAccountId(planAccount.getAccountId());
                param.setPlanId(planAccount.getPlanId());
                LongArticleSystemContentVO longArticleSystemContentVO = aigcService.getContentItemList(param);
                if (longArticleSystemContentVO == null || CollectionUtils.isEmpty(longArticleSystemContentVO.getContentItemList())) {
                    continue;
                }
                log.info("longArticleSystemContentVO totalCount={}", longArticleSystemContentVO.getTotalCount());
                for (ContentItemVO contentItemVO : longArticleSystemContentVO.getContentItemList()) {
                    MatchContent matchContent = new MatchContent();
                    matchContent.setSourceId(contentItemVO.getSourceId());
                    matchContent.setGhId(planAccount.getGhId());
                    matchContent.setPublishContentId(contentItemVO.getPublishContentId());
                    matchContent.setAccountName(planAccount.getAccountName());
                    matchContent.setContent(contentItemVO.getContent());
                    matchContent.setTitle(contentItemVO.getTitle());
                    matchContent.setFlowPoolLevelTag(contentItemVO.getFlowPoolLevelTag());
                    try {
                        queue.put(matchContent);
                    } catch (InterruptedException e) {
                        throw new RuntimeException(e);
                    }
                }
            }
        }).start();


        if (matchPoolExecutor.getCorePoolSize() - matchPoolExecutor.getActiveCount() > 0) {
            int threadSize = matchPoolExecutor.getCorePoolSize() - matchPoolExecutor.getActiveCount();
            // 启动消费者线程
            for (int i = 0; i < threadSize; i++) {
                matchPoolExecutor.submit(new Thread(() -> {
                    log.info("启动匹配小程序线程");
                    while (true) {
                        try {
                            // 超过 10 分钟没有数据，销毁当前线程
                            MatchContent matchContent = queue.poll(5, TimeUnit.MINUTES); // 等待最多 5 分钟
                            if (matchContent == null) {
                                break; // 退出当前线程
                            }
                            MatchVideo content = contentService.getContent(matchContent.getSourceId(), matchContent.getGhId(), 2);
                            if (content != null) {
                                Integer contentStatus = content.getContentStatus();
                                if (ContentStatusEnum.isSuccess(contentStatus)) {
                                    MatchMiniprogramStatusParam statusParam = new MatchMiniprogramStatusParam();
                                    statusParam.setStatus(2);
                                    statusParam.setPublishContentId(matchContent.getPublishContentId());
                                    aigcService.updateMatchMiniprogramStatus(statusParam);
                                    continue;
                                }
                                if (ContentStatusEnum.isFail(contentStatus)) {
                                    MatchMiniprogramStatusParam statusParam = new MatchMiniprogramStatusParam();
                                    statusParam.setStatus(3);
                                    statusParam.setPublishContentId(matchContent.getPublishContentId());
                                    String errorMessage = ContentStatusEnum.getErrorMessage(contentStatus);
                                    statusParam.setErrorMsg(errorMessage);
                                    aigcService.updateMatchMiniprogramStatus(statusParam);
                                }
                            } else {
                                MiniprogramCardRequest request = new MiniprogramCardRequest();
                                request.setGhId(matchContent.getGhId());
                                request.setAccountName(matchContent.getAccountName());
                                request.setContent(matchContent.getContent());
                                request.setTitle(matchContent.getTitle());
                                //请求到新服务
                                request.setStrategy("strategy_v2");
                                request.setArticleId(matchContent.getSourceId());
                                request.setFlowPoolLevelTag(matchContent.getFlowPoolLevelTag());
                                request.setPublishFlag(2);
                                matchService.matchMiniprogramVideo(request);
                            }
                        } catch (InterruptedException e) {
                            throw new RuntimeException(e);
                        }
                    }
                    log.info("启动匹配小程序线程结束");
                }));
            }
        }


    }


    private boolean effectiveTime(String startWindow, String endWindow, TimeZoneUtil.Timezone timezone) {
        if (!org.springframework.util.StringUtils.hasText(startWindow) || !org.springframework.util.StringUtils.hasText(endWindow)) {
            return true;
        }
        LocalTime currentTime = TimeZoneUtil
                .currentTime(Optional.ofNullable(timezone).orElse(TimeZoneUtil.Timezone.china));
        return currentTime.isAfter(LocalTime.parse(startWindow)) && currentTime.isBefore(LocalTime.parse(endWindow));
    }

    private boolean checkPlanAccount(PlanAccount planAccount) {
        boolean isGzhGroupPushPlan = Objects.nonNull(planAccount.getPushType())
                && (PublishGzhPushTypeEnum.auto_group_publish.getVal().equals(planAccount.getPushType())
                || PublishGzhPushTypeEnum.batch_group_publish.getVal().equals(planAccount.getPushType()));
        boolean flag = true;
        // 固定日期
        if (planAccount.getPublishRate() == 1) {
            List<Integer> weeks = JSON.parseArray(planAccount.getPublishDate(), Integer.class);
            if (!weeks.contains(TimeZoneUtil.getTodayDayOfWeek(TimeZoneUtil.Timezone.china))) {
                planAccount.setStatus(2);
                planAccountMapper.updateByPrimaryKeySelective(planAccount);
                return false;
            }
        }
        // 指定时间
        if (StringUtils.isNotEmpty(planAccount.getPublishPushTime())) {
            LocalTime currentTime = TimeZoneUtil.currentTime(TimeZoneUtil.Timezone.china);
            LocalTime pushTime = LocalTime.parse(planAccount.getPublishPushTime());
            if (currentTime.isBefore(pushTime.minusMinutes(20))) {
                return false;
            }
            if (currentTime.isAfter(pushTime.plusMinutes(360))) {
                return false;
            }
        } else if (StringUtils.isNotEmpty(planAccount.getPublishWindowStart())
                && StringUtils.isNotEmpty(planAccount.getPublishWindowEnd())) {
            if (!effectiveTime(planAccount.getPublishWindowStart(), planAccount.getPublishWindowEnd(),
                    TimeZoneUtil.Timezone.china)) {
                return false;
            }
            if (planAccount.getPublishTimeInterval() != null) {
                // 找到该账号最近发布的内容，判断时间间隔
                PublishContentExample example = new PublishContentExample();
                example.createCriteria().andStatusIn(Arrays.asList(1, 2))
                        .andCreateTimeGreaterThan(DateUtil.getThatDayDate())
                        .andPlanAccountIdEqualTo(planAccount.getId());
                example.setOrderByClause("create_time desc");
                List<PublishContent> publishContents = publishContentMapper.selectByExample(example);
                if (!CollectionUtils.isEmpty(publishContents)) {
                    long time = System.currentTimeMillis() - publishContents.get(0).getCreateTime().getTime();
                    if (time < planAccount.getPublishTimeInterval() * 60 * 1000L) {
                        return false;
                    }
                }
            }
        }
        PushRecordParam param = new PushRecordParam();
        param.setPlanId(planAccount.getPlanId());
        param.setPublishAccountId(planAccount.getAccountId());
        List<PushRecordVO> todayPushRecords = aigcService.getTodayPushRecords(param);


        //查询aigc发布成功数量
        int aigcCount = 0;
        if (!CollectionUtils.isEmpty(todayPushRecords)) {
            List<PushRecordVO> collect = todayPushRecords.stream()
                    .filter(e -> ((e.getPushStatus() != 3) && (e.getPublishStatus() != 3))).collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(collect)) {
                for (PushRecordVO pushRecordVO : collect) {
                    if (pushRecordVO.getPublishContentCount() != null) {
                        aigcCount += pushRecordVO.getPublishContentCount();
                    }
                }
            }
        }
        //长文系统发布数量
        int sendCount = publicContentService.getSendCount(planAccount.getId());
        //取发布数量多的
        sendCount = Math.max(aigcCount, sendCount);
        if (isGzhGroupPushPlan) {
            //公众号群发每天只能成功发布一次
            if (sendCount >= 1) {
                return false;
            }
            //公众号群发每天最多尝试3次
            if (planAccount.getRetryCount() >= 3) {
                planAccount.setStatus(3);
                planAccountMapper.updateByPrimaryKeySelective(planAccount);
                LarkRobotUtil.sendMessage("计划发布失败3次，请查看 账号计划id为" + planAccount.getPlanId());
                return false;
            }
        }
        Integer publishNum = planAccount.getPublishNum();
        if (publishNum == null || publishNum == 0) {
            LarkRobotUtil.sendMessage("发布数量设置为0 planAccountId=" + planAccount.getId());
            return false;
        }
        if (sendCount >= publishNum) {
            return false;
        }

        int todayNeedSendCount = publishNum - sendCount;
        // 本次推送数量
        Integer prePushNum = planAccount.getPublishPreNum();
        if (Objects.isNull(prePushNum)) {
            prePushNum = 1;
        }
        //如果预计推送数量大于剩余推送数量  本次数量设置为剩余推送数量
        if (prePushNum > todayNeedSendCount) {
            prePushNum = todayNeedSendCount;
        }
        // 自动发布消息，每次暂时限定发布一条
        if (PublishGzhPushTypeEnum.robopost.getVal().equals(planAccount.getPushType())) {
            prePushNum = 1;
        }
        // 本次最少推送数量
        Integer preMinPushNum = planAccount.getPublishPreMinNum();
        if (Objects.isNull(preMinPushNum)) {
            preMinPushNum = prePushNum;
        }
        if (preMinPushNum > prePushNum) {
            preMinPushNum = prePushNum;
        }
        planAccount.setPublishPreNum(prePushNum);
        planAccount.setPublishPreMinNum(preMinPushNum);
        return flag;
    }


    @Override
    public void core(String param) throws InterruptedException {

        if (corePoolExecutor.getCorePoolSize() - corePoolExecutor.getActiveCount() > 0) {
            int threadSize = corePoolExecutor.getCorePoolSize() - corePoolExecutor.getActiveCount();
            log.info("threadNum={}", threadSize);
            CountDownLatch latch = new CountDownLatch(threadSize);
            // 启动消费者线程
            for (int i = 0; i < threadSize; i++) {
                matchPoolExecutor.submit(new Thread(() -> {
                    log.info("启动核心操作线程");
                    while (true) {
                        try {
                            // 超过 5 分钟没有数据，销毁当前线程
                            PlanAccount planAccount = queue1.poll(5, TimeUnit.MINUTES); // 等待最多 5 分钟
                            if (planAccount == null) {
                                break; // 退出当前线程
                            }
                            operatePlanAccount(planAccount);
                        } catch (Exception e) {
                            log.error("", e);
                        }
                    }
                    log.info("核心操作线程结束");
                    latch.countDown();
                }));
            }
            List<PlanAccount> planAccounts;
            if (StringUtils.isEmpty(param)) {
                planAccounts = planAccountService.getNormalPlanAccount();
            } else {
                planAccounts = planAccountService.getPlanAccount(param);
            }
            if (CollectionUtils.isEmpty(planAccounts)) {
                return;
            }
            for (PlanAccount planAccount : planAccounts) {
                boolean flag = checkPlanAccount(planAccount);
                if (!flag && StringUtils.isEmpty(param)) {
                    continue;
                }
                queue1.put(planAccount);
            }
            latch.await();
        }
    }

    private void operatePlanAccount(PlanAccount planAccount) {
        boolean flag = checkPlanAccount(planAccount);
        if (!flag) {
            return;
        }
        //获取待排序数据  数量不足会返回空list
        List<PublishArticleData> rankList = contentService.getWaitingSort(planAccount);
        if (CollectionUtils.isEmpty(rankList)) {
            return;
        }
        if (StringUtils.isNotEmpty(planAccount.getSortStrategy())) {
            rankList = contentService.getSortList(planAccount);
            if (CollectionUtils.isEmpty(rankList)) {
                return;
            }
        }
        //文章处理  小程序查询和发布
        List<PushContentParam> pushContentList = new ArrayList<>();
        List<Long> sendIds = new ArrayList<>();
        for (PublishArticleData publishArticleData : rankList) {
            PushContentParam contentParam = new PushContentParam();
            PublishContent publishContent = publicContentService.getPublishContent(planAccount, publishArticleData);
            if (publishContent == null) {
                continue;
            }
            //获取小程序
            List<PublishMiniprogram> publishMiniprogramList = publicContentService.getPublishMiniprograms(publishArticleData);
            //不存在则重新生成
            if (CollectionUtils.isEmpty(publishMiniprogramList)) {
                MatchVideo matchVideo = contentService.getMatchVideo(publishContent, planAccount);
                if (matchVideo == null) {
                    continue;
                }
                List<VideoDetail> videoDetails = contentService.getPublishVideoDetail(publishContent, planAccount, matchVideo);
                log.info("publishContentId={}, videoDetails={}", publishContent.getId(), videoDetails);
                if (CollectionUtils.isEmpty(videoDetails)) {
                    continue;
                }
                //获取小程序卡片  判断封面是否可用
                List<PublishMiniprogramBo> publishMiniprogramBoList = cardService.generateCards(videoDetails, planAccount, publishContent);
                if (CollectionUtils.isEmpty(publishMiniprogramBoList) && publishMiniprogramBoList.size() < 2) {
                    continue;
                }
                for (PublishMiniprogramBo publishMiniprogramBo : publishMiniprogramBoList) {
                    //插入rootSource
                    rootSourceService.addRootSource(publishMiniprogramBo, planAccount, publishArticleData);

                    //拷贝小程序BO到PO中 写入数据库
                    PublishMiniprogram publishMiniprogram = new PublishMiniprogram();
                    BeanUtils.copyProperties(publishMiniprogramBo, publishMiniprogram);
                    publishMiniprogramMapper.insertSelective(publishMiniprogram);
                    publishMiniprogramList.add(publishMiniprogram);
                }
                contentService.updateMatchContent(publishContent, publishMiniprogramBoList, matchVideo);
            }
            sendIds.add(publishContent.getId());
            log.info("publishMiniprogramList={}", publishMiniprogramList);
            List<PublishMiniprogramParam> publishCardList = getPublishCardList(publishMiniprogramList);
            contentParam.setPublishContentId(publishContent.getPublishContentId());
            contentParam.setMiniprogramCardList(publishCardList);
            pushContentList.add(contentParam);
        }
        CreatePushTaskParam gzhPushParam = getCreatePushTaskParam(planAccount, pushContentList);
        if (gzhPushParam == null) {
            return;
        }
        log.info("gzhPushParam={}", gzhPushParam);
        String pushId = aigcService.createPushTask(gzhPushParam);
        log.info("pushId = {}", pushId);
        if (StringUtils.isNotEmpty(pushId)) {
            publicContentService.updatePublishContentPushId(sendIds, pushId);
        }
    }

    private CreatePushTaskParam getCreatePushTaskParam(PlanAccount planAccount, List<PushContentParam> pushContentList) {
        CreatePushTaskParam gzhPushParam = new CreatePushTaskParam();
        gzhPushParam.setPlanId(planAccount.getPlanId());
        gzhPushParam.setAccountId(planAccount.getAccountId());
        gzhPushParam.setPushType(planAccount.getPushType());
        gzhPushParam.setPushScheduleTimestamp(System.currentTimeMillis());
        if (StringUtils.isNotEmpty(planAccount.getPublishPushTime())) {
            String dateTimeStr = TimeZoneUtil.todayYMD(TimeZoneUtil.Timezone.china) + " " + planAccount.getPublishPushTime();
            gzhPushParam.setPushScheduleTimestamp(TimeZoneUtil.getDateStrTimestamp(dateTimeStr, "yyyy-MM-dd HH:mm:ss", TimeZoneUtil.Timezone.china));
        } else {
            gzhPushParam.setPushScheduleTimestamp(System.currentTimeMillis());
        }
        //文章列表为空直接返回
        if (CollectionUtils.isEmpty(pushContentList)) {
            return null;
        }
        //自动群发少于最少数量不发送
        if (Objects.equals(planAccount.getPushType(), PublishGzhPushTypeEnum.auto_group_publish.getVal())
                && pushContentList.size() < planAccount.getPublishPreMinNum()) {
            return null;
        }
        gzhPushParam.setPushContentList(pushContentList);
        return gzhPushParam;
    }

    private List<PublishMiniprogramParam> getPublishCardList(List<PublishMiniprogram> publishMiniprogramList) {
        List<PublishMiniprogramParam> publishCardList = new ArrayList<>();
        for (PublishMiniprogram publishMiniprogram : publishMiniprogramList) {
            PublishMiniprogramParam param = new PublishMiniprogramParam();
            param.setProgramId(publishMiniprogram.getAppId());
            param.setProgramName(publishMiniprogram.getAppName());
            param.setProgramAvatar(publishMiniprogram.getAvatar());
            param.setMini_id(publishMiniprogram.getMiniId());
            param.setMiniprogramUseType(publishMiniprogram.getMiniProgramType());
            String videoTitle = publishMiniprogram.getVideoTitle();
            if (org.springframework.util.StringUtils.hasText(videoTitle)) {
                videoTitle = videoTitle.replace("\"", "");
                if (videoTitle.length() > 32) {
                    videoTitle = videoTitle.substring(0, 32);
                }
                param.setProductionName(videoTitle);
            }
            param.setProductionCover(publishMiniprogram.getWxUrl());
            param.setProductionPath(publishMiniprogram.getProductionPath());
            param.setRootShareId(publishMiniprogram.getRootShareId());
            param.setSource(publishMiniprogram.getSource());
            param.setVideoUrl(publishMiniprogram.getVideoPath());
            param.setParagraphPosition(BigDecimal.valueOf(publishMiniprogram.getIndex() * 0.25));
            if (publishMiniprogram.getIndex() == 1) {
                param.setParagraphPosition(BigDecimal.valueOf(0.01));
            }
            param.setTraceId(publishMiniprogram.getTraceId());
            publishCardList.add(param);
        }
        return publishCardList;
    }

    @Override
    public void getPushStatus() {
        PublishContentExample example = new PublishContentExample();
        example.createCriteria().andStatusEqualTo(1).andPushIdIsNotNull();
        List<PublishContent> publishContents = publishContentMapper.selectByExample(example);
        Map<String, List<PublishContent>> groupedByPushId = publishContents.stream()
                .filter(content -> content.getPushId() != null) // 过滤掉 pushId 为空的情况
                .collect(Collectors.groupingBy(PublishContent::getPushId));

        for (Map.Entry<String, List<PublishContent>> entry : groupedByPushId.entrySet()) {
            String pushId = entry.getKey();
            PushStatusParam param = new PushStatusParam();
            param.setPushId(pushId);
            PushStatusVO pushStatusVO = aigcService.queryPushStatus(param);
            log.info("pushStatusVO={} param={}", pushStatusVO, param);
            if (pushStatusVO == null) {
                continue;
            }
            int updateStatus = 0;
            //推送失败
            if (pushStatusVO.getPushStatus() == 3) {
                updateStatus = 3;
            }
            //推送成功
            if (pushStatusVO.getPushStatus() == 2) {
                //发表失败
                if (pushStatusVO.getPublishStatus() == 3) {
                    updateStatus = 3;
                    if (StringUtils.isNotEmpty(pushStatusVO.getErrorCode())) {
                        int errorCode = Integer.parseInt(pushStatusVO.getErrorCode());
                        if (errorCode == 6 || errorCode == 7 || errorCode == 8 || errorCode == 999) {
                            updateStatus = 2;
                        }
                    }

                }
                //发表成功
                if (pushStatusVO.getPublishStatus() == 2) {
                    updateStatus = 2;
                }
            }


            Long planAccountId = entry.getValue().get(0).getPlanAccountId();
            if (planAccountId == null) {
                continue;
            }
            PlanAccount planAccount = planAccountService.getPlanAccountById(planAccountId);
            if (updateStatus == 2) {
                for (PublishContent publishContent : entry.getValue()) {
                    publicContentService.updatePublishContentStatus(updateStatus, publishContent.getId(), pushStatusVO.getErrorMsg());
//                    if (StringUtils.isNotEmpty(publishContent.getTraceId())) {
//                        videoService.miniProgramVideoOff(publishContent.getTraceId());
//                    } else {
//                        LarkRobotUtil.sendMessage("publishContent traceId is null publishContent id=" + publishContent.getId());
//                    }
                }
                if (planAccount == null) {
                    continue;
                }
                if (Objects.equals(planAccount.getPushType(), PublishGzhPushTypeEnum.auto_group_publish.getVal())) {
                    planAccountService.updateStatus(2, planAccountId);
                }
            }
            if (updateStatus == 3) {
                PublishContentExample publishContentExample = new PublishContentExample();
                publishContentExample.createCriteria().andPushIdEqualTo(pushId);
                PublishContent update = new PublishContent();
                if (StringUtils.isNotEmpty(pushStatusVO.getErrorMsg())) {
                    update.setReason(pushStatusVO.getErrorMsg());
                }
                update.setStatus(updateStatus);
                publishContentMapper.updateByExampleSelective(update, publishContentExample);
                if (planAccount == null) {
                    continue;
                }
                planAccountService.updateRetry(planAccount);
                if (Objects.equals(planAccount.getPushType(), PublishGzhPushTypeEnum.auto_group_publish.getVal())) {
                    if (StringUtils.isNotEmpty(pushStatusVO.getErrorMsg())) {
                        //45028群发配额用完不再重试  48004封号账号也不再重试
                        if (pushStatusVO.getErrorMsg().contains("45028") || pushStatusVO.getErrorMsg().contains("48004")) {
                            planAccountService.updateStatus(3, planAccountId);
                        }
                    }
                }
            }
            if (updateStatus == 0) {
                //查询创建时间   重试超过2h 直接设置失败
                long createTimestamp = entry.getValue().get(0).getCreateTime().getTime();
                long nowTimestamp = System.currentTimeMillis();
                if (nowTimestamp - createTimestamp > TimeConstant.MILLISECOND_HOUR * 2) {
                    if (StringUtils.isNotEmpty(pushStatusVO.getErrorMsg())) {
                        PublishContentExample publishContentExample = new PublishContentExample();
                        publishContentExample.createCriteria().andPushIdEqualTo(pushId);
                        PublishContent update = new PublishContent();
                        if (StringUtils.isNotEmpty(pushStatusVO.getErrorMsg())) {
                            update.setReason(pushStatusVO.getErrorMsg());
                        }
                        update.setStatus(3);
                        publishContentMapper.updateByExampleSelective(update, publishContentExample);
                        planAccountService.updateRetry(planAccount);
                        LarkRobotUtil.sendMessage("重试超过2h失败，pushId=", pushId);
                    } else {
                        long hour = (nowTimestamp - createTimestamp) / 1000 / 3600;
                        LarkRobotUtil.sendMessage("重试时间：" + hour + "pushId=", pushId);
                    }
                }
            }
            if (StringUtils.isNotEmpty(pushStatusVO.getErrorMsg()) && !pushStatusVO.getErrorMsg().contains("45028")) {
                log.error("push error pushId={} msg={}", pushId, pushStatusVO.getErrorMsg());
                String message = String.format("发布失败，请查看，pushId=%s 失败信息：%s 账号名称：%s 计划名称：%s",
                        pushId, pushStatusVO.getErrorMsg(), planAccount.getAccountName(), planAccount.getPlanName());
                LarkRobotUtil.sendMessage(message);
            }
        }
    }
}
